@@ -30,8 +30,10 @@ Encoding basic Python object hierarchies::
 Compact encoding::
 
     >>> import simplejson
-    >>> simplejson.dumps([1,2,3,{'4': 5, '6': 7}], separators=(',',':'))
-    '[1,2,3,{"4":5,"6":7}]'
+    >>> compact = simplejson.dumps([1,2,3,{'4': 5, '6': 7}], separators=(',',':'))
+    >>> # Can't assume dict ordering
+    >>> compact in ('[1,2,3,{"4":5,"6":7}]', '[1,2,3,{"6":7,"4":5}]')
+    True
 
 Pretty printing::
 
@@ -89,23 +91,25 @@ Extending JSONEncoder::
 Using simplejson from the shell to validate and
 pretty-print::
     
-    $ echo '{"json":"obj"}' | python -msimplejson
+    $ echo '{"json":"obj"}' | python -msimplejson.tool
     {
         "json": "obj"
     }
-    $ echo '{ 1.2:3.4}' | python -msimplejson
+    $ echo '{ 1.2:3.4}' | python -msimplejson.tool
     Expecting property name: line 1 column 2 (char 2)
 
 Note that the JSON produced by this module's default settings
 is a subset of YAML, so it may be used as a serializer for that as well.
 """
-__version__ = '1.9'
+__version__ = '1.9.3'
 __all__ = [
     'dump', 'dumps', 'load', 'loads',
     'JSONDecoder', 'JSONEncoder',
 ]
 
 if __name__ == '__main__':
+    import warnings
+    warnings.warn('python -msimplejson is deprecated, use python -msiplejson.tool', DeprecationWarning)
     from simplejson.decoder import JSONDecoder
     from simplejson.encoder import JSONEncoder
 else:
@@ -369,32 +373,6 @@ def write(obj):
     return dumps(obj)
 
 
-#
-# Pretty printer:
-#     curl http://mochikit.com/examples/ajax_tables/domains.json | python -msimplejson
-#
-
-
-def main():
-    import sys
-    if len(sys.argv) == 1:
-        infile = sys.stdin
-        outfile = sys.stdout
-    elif len(sys.argv) == 2:
-        infile = open(sys.argv[1], 'rb')
-        outfile = sys.stdout
-    elif len(sys.argv) == 3:
-        infile = open(sys.argv[1], 'rb')
-        outfile = open(sys.argv[2], 'wb')
-    else:
-        raise SystemExit("%s [infile [outfile]]" % (sys.argv[0],))
-    try:
-        obj = load(infile)
-    except ValueError, e:
-        raise SystemExit(e)
-    dump(obj, outfile, sort_keys=True, indent=4)
-    outfile.write('\n')
-
-
 if __name__ == '__main__':
-    main()
+    import simplejson.tool
+    simplejson.tool.main()
